@@ -44,7 +44,6 @@
 #include "gpk-gnome.h"
 #include "gpk-dialog.h"
 #include "gpk-error.h"
-#include "gpk-consolekit.h"
 #include "gpk-cell-renderer-size.h"
 #include "gpk-cell-renderer-info.h"
 #include "gpk-cell-renderer-restart.h"
@@ -79,6 +78,7 @@ static PkRestartEnum restart_update = PK_RESTART_ENUM_NONE;
 static guint size_total = 0;
 static GConfClient *gconf_client = NULL;
 static gchar **install_package_ids = NULL;
+static EggConsoleKit *console = NULL;
 
 enum {
 	GPK_UPDATES_COLUMN_TEXT,
@@ -89,6 +89,7 @@ enum {
 	GPK_UPDATES_COLUMN_CLICKABLE,
 	GPK_UPDATES_COLUMN_RESTART,
 	GPK_UPDATES_COLUMN_SIZE,
+	GPK_UPDATES_COLUMN_SIZE_DISPLAY,
 	GPK_UPDATES_COLUMN_PERCENTAGE,
 	GPK_UPDATES_COLUMN_STATUS,
 	GPK_UPDATES_COLUMN_DETAILS_OBJ,
@@ -138,18 +139,15 @@ out:
 static void
 gpk_update_viewer_shutdown (void)
 {
-	EggConsoleKit *console;
 	GError *error = NULL;
 	gboolean ret;
 
 	/* use consolekit to restart */
-	console = egg_console_kit_new ();
 	ret = egg_console_kit_restart (console, &error);
 	if (!ret) {
 		egg_warning ("cannot restart: %s", error->message);
 		g_error_free (error);
 	}
-	g_object_unref (console);
 }
 
 /**
@@ -434,7 +432,11 @@ gpk_update_viewer_button_install_cb (GtkWidget *widget, gpointer data)
 	install_package_ids = g_strdupv (package_ids);
 
 	/* get packages that also have to be updated */
+#if PK_CHECK_VERSION(0,5,2)
+	ret = pk_client_simulate_update_packages (client_primary, package_ids, &error);
+#else
 	ret = pk_client_get_depends (client_primary, pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), package_ids, TRUE, &error);
+#endif
 	if (!ret) {
 		egg_warning ("cannot get depends for updates: %s", error->message);
 		g_error_free (error);
@@ -529,7 +531,7 @@ gpk_update_viewer_details_cb (PkClient *client, const PkDetailsObj *obj, gpointe
 
 	path = gpk_update_viewer_model_get_path (model, obj->id);
 	if (path == NULL) {
-		egg_debug ("not found ID for group");
+		egg_debug ("not found ID for details");
 		return;
 	}
 
@@ -537,7 +539,9 @@ gpk_update_viewer_details_cb (PkClient *client, const PkDetailsObj *obj, gpointe
 	gtk_tree_path_free (path);
 	gtk_list_store_set (list_store_updates, &iter,
 			    GPK_UPDATES_COLUMN_DETAILS_OBJ, (gpointer) pk_details_obj_copy (obj),
-			    GPK_UPDATES_COLUMN_SIZE, (gint)obj->size, -1);
+			    GPK_UPDATES_COLUMN_SIZE, (gint)obj->size,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, (gint)obj->size,
+			    -1);
 	/* in cache */
 	if (obj->size == 0)
 		gtk_list_store_set (list_store_updates, &iter,
@@ -578,9 +582,11 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 	GtkTreeView *treeview;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
+	GtkTreeViewColumn *column;
 	GtkWidget *widget;
 	GtkTreePath *path;
 	gboolean selected;
+	gboolean scroll;
 
 	pk_client_get_role (client, &role, NULL, NULL);
 	egg_debug ("role = %s, package = %s:%s:%s", pk_role_enum_to_text (role),
@@ -590,10 +596,17 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 	package_id = pk_package_id_to_string (obj->id);
 
 	/* are we simulating to get deps? */
+#if PK_CHECK_VERSION(0,5,2)
+	if (role == PK_ROLE_ENUM_SIMULATE_UPDATE_PACKAGES) {
+		egg_debug ("ignoring %s as we are simulating", package_id);
+		goto out;
+	}
+#else
 	if (role == PK_ROLE_ENUM_GET_DEPENDS) {
 		egg_debug ("ignoring %s as we are in the depends phase", package_id);
 		goto out;
 	}
+#endif
 
 	/* used for progress */
 	if (!gpk_update_viewer_is_update_info (obj->info)) {
@@ -613,6 +626,13 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 
 		gtk_tree_model_get_iter (model, &iter, path);
 
+		/* scroll to the active cell */
+		scroll = gconf_client_get_bool (gconf_client, GPK_CONF_UPDATE_VIEWER_SCROLL_ACTIVE, NULL);
+		if (scroll) {
+			column = gtk_tree_view_get_column (treeview, 3);
+			gtk_tree_view_scroll_to_cell (treeview, path, column, FALSE, 0.0f, 0.0f);
+		}
+
 		/* if the info is finished, change the status to past tense */
 		if (obj->info == PK_INFO_ENUM_FINISHED) {
 			gtk_tree_model_get (model, &iter,
@@ -626,10 +646,6 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 		gtk_list_store_set (list_store_updates, &iter,
 				    GPK_UPDATES_COLUMN_STATUS, info, -1);
 
-		/* set size zero when we get a info update */
-		gtk_list_store_set (list_store_updates, &iter,
-				    GPK_UPDATES_COLUMN_SIZE, 0, -1);
-
 		gtk_tree_path_free (path);
 
 		/* set package description */
@@ -653,6 +669,7 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 			    GPK_UPDATES_COLUMN_RESTART, PK_RESTART_ENUM_NONE,
 			    GPK_UPDATES_COLUMN_STATUS, PK_INFO_ENUM_UNKNOWN,
 			    GPK_UPDATES_COLUMN_SIZE, 0,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, 0,
 			    GPK_UPDATES_COLUMN_PERCENTAGE, 0,
 			    GPK_UPDATES_COLUMN_PULSE, -1,
 			    -1);
@@ -830,14 +847,23 @@ gpk_update_viewer_reconsider_info (GtkTreeModel *model)
 	if (number_total == 0) {
 		gtk_label_set_label (GTK_LABEL (widget), "");
 	} else {
-		text_size = g_format_size_for_display (size_total);
-		/* TRANSLATORS: how many updates are selected in the UI */
-		text = g_strdup_printf (ngettext ("%i update selected (%s)",
-						  "%i updates selected (%s)",
-						  number_total), number_total, text_size);
-		gtk_label_set_label (GTK_LABEL (widget), text);
-		g_free (text);
-		g_free (text_size);
+		if (size_total == 0) {
+			/* TRANSLATORS: how many updates are selected in the UI */
+			text = g_strdup_printf (ngettext ("%i update selected",
+							  "%i updates selected",
+							  number_total), number_total);
+			gtk_label_set_label (GTK_LABEL (widget), text);
+			g_free (text);
+		} else {
+			text_size = g_format_size_for_display (size_total);
+			/* TRANSLATORS: how many updates are selected in the UI, and the size of packages to download */
+			text = g_strdup_printf (ngettext ("%i update selected (%s)",
+							  "%i updates selected (%s)",
+							  number_total), number_total, text_size);
+			gtk_label_set_label (GTK_LABEL (widget), text);
+			g_free (text);
+			g_free (text_size);
+		}
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_summary"));
@@ -1124,7 +1150,7 @@ gpk_update_viewer_treeview_add_columns_update (GtkTreeView *treeview)
 	/* TRANSLATORS: a column that has size of each package */
 	gtk_tree_view_column_set_title (column, _("Size"));
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), FALSE);
-	gtk_tree_view_column_set_sort_column_id (column, GPK_UPDATES_COLUMN_SIZE);
+	gtk_tree_view_column_set_sort_column_id (column, GPK_UPDATES_COLUMN_SIZE_DISPLAY);
 
 	/* size */
 	renderer = gpk_cell_renderer_size_new ();
@@ -1133,12 +1159,12 @@ gpk_update_viewer_treeview_add_columns_update (GtkTreeView *treeview)
 		      "xalign", 1.0f,
 		      NULL);
 	g_object_set (renderer,
-		      "value", GPK_UPDATES_COLUMN_SIZE, NULL);
+		      "value", GPK_UPDATES_COLUMN_SIZE_DISPLAY, NULL);
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_SIZE);
+	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_SIZE_DISPLAY);
 
 	gtk_tree_view_append_column (treeview, column);
-	g_object_set_data (G_OBJECT (column), "tooltip-id", GINT_TO_POINTER (GPK_UPDATES_COLUMN_SIZE));
+	g_object_set_data (G_OBJECT (column), "tooltip-id", GINT_TO_POINTER (GPK_UPDATES_COLUMN_SIZE_DISPLAY));
 }
 
 /**
@@ -1543,6 +1569,7 @@ gpk_update_viewer_check_restart (PkRestartEnum restart)
 	const gchar *message;
 	const gchar *button;
 	GtkResponseType response;
+	gboolean show_button = TRUE;
 
 	/* get the text */
 	title = gpk_restart_enum_to_localised_text (restart);
@@ -1551,11 +1578,28 @@ gpk_update_viewer_check_restart (PkRestartEnum restart)
 		message = _("Some of the updates that were installed require the computer to be restarted before the changes will be applied.");
 		/* TRANSLATORS: the button text for the restart */
 		button = _("Restart Computer");
-	} else {
+
+	} else if (restart == PK_RESTART_ENUM_SECURITY_SYSTEM) {
+		/* TRANSLATORS: the message text for the restart */
+		message = _("Some of the updates that were installed require the computer to be restarted to remain secure.");
+		/* TRANSLATORS: the button text for the restart */
+		button = _("Restart Computer");
+
+	} else if (restart == PK_RESTART_ENUM_SESSION) {
 		/* TRANSLATORS: the message text for the logout */
 		message = _("Some of the updates that were installed require you to log out and back in before the changes will be applied.");
 		/* TRANSLATORS: the button text for the logout */
 		button = _("Log Out");
+
+	} else if (restart == PK_RESTART_ENUM_SECURITY_SESSION) {
+		/* TRANSLATORS: the message text for the logout */
+		message = _("Some of the updates that were installed require you to log out and back in to remain secure.");
+		/* TRANSLATORS: the button text for the logout */
+		button = _("Log Out");
+
+	} else {
+		egg_warning ("unknown restart enum");
+		goto out;
 	}
 
 	/* show modal dialog */
@@ -1563,7 +1607,16 @@ gpk_update_viewer_check_restart (PkRestartEnum restart)
 	dialog = gtk_message_dialog_new (window, GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
 					 "%s", title);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+
+	/* check to see if restart is possible */
+	if (restart == PK_RESTART_ENUM_SYSTEM ||
+	    restart == PK_RESTART_ENUM_SECURITY_SYSTEM) {
+		egg_console_kit_can_restart (console, &show_button, NULL);
+	}
+
+	/* only show the button if we can do the action */
+	if (show_button)
+		gtk_dialog_add_button (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG(dialog), "%s", message);
 	gtk_window_set_icon_name (GTK_WINDOW(dialog), GPK_ICON_SOFTWARE_INSTALLER);
 
@@ -1662,6 +1715,7 @@ gpk_update_viewer_deps_update_event_cb (GpkHelperDepsUpdate *helper, GtkResponse
 					    GPK_UPDATES_COLUMN_RESTART, PK_RESTART_ENUM_NONE,
 					    GPK_UPDATES_COLUMN_STATUS, PK_INFO_ENUM_UNKNOWN,
 					    GPK_UPDATES_COLUMN_SIZE, 0,
+					    GPK_UPDATES_COLUMN_SIZE_DISPLAY, 0,
 					    GPK_UPDATES_COLUMN_PERCENTAGE, 0,
 					    GPK_UPDATES_COLUMN_PULSE, -1,
 					    -1);
@@ -1756,8 +1810,11 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 	}
 
 	/* finished depends check, show any extras */
-	if (role == PK_ROLE_ENUM_GET_DEPENDS) {
-
+#if PK_CHECK_VERSION(0,5,2)
+	if (exit == PK_EXIT_ENUM_SUCCESS && role == PK_ROLE_ENUM_SIMULATE_UPDATE_PACKAGES) {
+#else
+	if (exit == PK_EXIT_ENUM_SUCCESS && role == PK_ROLE_ENUM_GET_DEPENDS) {
+#endif
 		/* show deps dialog */
 		list = pk_client_get_package_list (client);
 		gpk_helper_deps_update_show (helper_deps_update, list);
@@ -1854,7 +1911,9 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 
 		/* check restart */
 		if (restart_update == PK_RESTART_ENUM_SYSTEM ||
-		    restart_update == PK_RESTART_ENUM_SESSION) {
+		    restart_update == PK_RESTART_ENUM_SESSION ||
+		    restart_update == PK_RESTART_ENUM_SECURITY_SESSION ||
+		    restart_update == PK_RESTART_ENUM_SECURITY_SYSTEM) {
 			gpk_update_viewer_check_restart (restart_update);
 			g_main_loop_quit (loop);
 		}
@@ -1909,6 +1968,8 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 	/* we pressed cancel */
 	if (exit != PK_EXIT_ENUM_SUCCESS) {
 		gpk_update_viewer_undisable_packages ();
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_install"));
+		gtk_widget_set_sensitive (widget, TRUE);
 	}
 }
 
@@ -2027,6 +2088,8 @@ gpk_update_viewer_progress_changed_cb (PkClient *client, guint percentage, guint
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	guint oldval;
+	guint size;
+	guint size_display;
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "progressbar_progress"));
 	gtk_widget_show (widget);
@@ -2048,15 +2111,22 @@ gpk_update_viewer_progress_changed_cb (PkClient *client, guint percentage, guint
 	}
 
 	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_model_get (model, &iter, GPK_UPDATES_COLUMN_PERCENTAGE, &oldval, -1);
+	gtk_tree_model_get (model, &iter,
+			    GPK_UPDATES_COLUMN_PERCENTAGE, &oldval,
+			    GPK_UPDATES_COLUMN_SIZE, &size,
+			    -1);
 	if ((oldval > 0 && oldval < 100) != (subpercentage > 0 && subpercentage < 100)) {
 		if (oldval > 0 && oldval < 100)
 			gpk_update_viewer_remove_active_row (model, path);
 		else
 			gpk_update_viewer_add_active_row (model, path);
 	}
+
+	size_display = size - ((size * subpercentage) / 100);
 	gtk_list_store_set (list_store_updates, &iter,
-			    GPK_UPDATES_COLUMN_PERCENTAGE, subpercentage, -1);
+			    GPK_UPDATES_COLUMN_PERCENTAGE, subpercentage,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, size_display,
+			    -1);
 
 	gtk_tree_path_free (path);
 }
@@ -2072,14 +2142,20 @@ gpk_update_viewer_error_code_cb (PkClient *client, PkErrorCodeEnum code, const g
 	/* ignore some errors */
 	if (code == PK_ERROR_ENUM_PROCESS_KILL ||
 	    code == PK_ERROR_ENUM_TRANSACTION_CANCELLED) {
-		egg_debug ("error ignored %s\n%s", pk_error_enum_to_text (code), details);
+		egg_debug ("error ignored %s: %s", pk_error_enum_to_text (code), details);
 		return;
 	}
 
 	/* ignore the ones we can handle */
 	if (code == PK_ERROR_ENUM_NO_LICENSE_AGREEMENT ||
 	    pk_error_code_is_need_untrusted (code)) {
-		egg_debug ("error ignored as we're handling %s\n%s", pk_error_enum_to_text (code), details);
+		egg_debug ("error ignored as we're handling %s: %s", pk_error_enum_to_text (code), details);
+		return;
+	}
+
+	/* ignore not authorised, which seems odd but this will happen if the user clicks cancel */
+	if (code == PK_ERROR_ENUM_NOT_AUTHORIZED) {
+		egg_debug ("auth failure '%s' ignored: %s", pk_error_enum_to_text (code), details);
 		return;
 	}
 
@@ -2333,6 +2409,7 @@ gpk_update_viewer_get_new_update_list (void)
 	GError *error = NULL;
 	GtkWidget *widget;
 	gchar *text = NULL;
+	PkBitfield filter = PK_FILTER_ENUM_NONE;
 
 	/* clear all widgets */
 	gtk_list_store_clear (list_store_updates);
@@ -2351,8 +2428,15 @@ gpk_update_viewer_get_new_update_list (void)
 		goto out;
 	}
 
+	/* only show newest updates? */
+	ret = gconf_client_get_bool (gconf_client, GPK_CONF_UPDATE_VIEWER_ONLY_NEWEST, NULL);
+	if (ret) {
+		egg_warning ("only showing newest updates");
+		filter = pk_bitfield_from_enums (PK_FILTER_ENUM_NEWEST, -1);
+	}
+
 	/* get new list */
-	ret = pk_client_get_updates (client_primary, pk_bitfield_from_enums (PK_FILTER_ENUM_NEWEST, -1), &error);
+	ret = pk_client_get_updates (client_primary, filter, &error);
 	if (!ret) {
 		egg_warning ("Failed to get updates: %s", error->message);
 		g_error_free (error);
@@ -2793,6 +2877,7 @@ main (int argc, char *argv[])
 
 	/* get GConf instance */
 	gconf_client = gconf_client_get_default ();
+	console = egg_console_kit_new ();
 
 	g_signal_connect (unique_app, "message-received", G_CALLBACK (gpk_update_viewer_message_received_cb), NULL);
 
@@ -2872,7 +2957,8 @@ main (int argc, char *argv[])
 	/* create list stores */
 	list_store_updates = gtk_list_store_new (GPK_UPDATES_COLUMN_LAST, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
 						 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
-						 G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_INT);
+						 G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT,
+						 G_TYPE_UINT, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_INT);
 	text_buffer = gtk_text_buffer_new (NULL);
 	gtk_text_buffer_create_tag (text_buffer, "para",
 				    "pixels_above_lines", 5,
@@ -3011,6 +3097,7 @@ out_build:
 	g_object_unref (markdown);
 	g_object_unref (client_primary);
 	g_object_unref (client_secondary);
+	g_object_unref (console);
 	g_object_unref (builder);
 unique_out:
 	g_object_unref (unique_app);
